{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import libraries"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import ee\n",
    "import geemap"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Create an interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map = geemap.Map(center=[40, -100], zoom=4)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Add Earth Engine Python script"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Add Earth Engine dataset\n",
    "image = ee.Image(\"USGS/SRTMGL1_003\")\n",
    "\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "#  Chapter:      A2.5 Water Balance and Drought\n",
    "#  Checkpoint:   A25s4\n",
    "#  Author:       Ate Poortinga, Quyen Nguyen, Nyein Soe Thwal, Andréa Puzzi Nicolau\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "\n",
    "# Import the Lower Mekong boundary.\n",
    "mekongBasin = ee.FeatureCollection(\n",
    "    'projects/gee-book/assets/A2-5/lowerMekongBasin')\n",
    "\n",
    "Map.centerObject(mekongBasin, 6)\n",
    "\n",
    "classStruct = {\n",
    "    'unknown': {\n",
    "        'number': 0,\n",
    "        'color': '6f6f6f'\n",
    "    },\n",
    "    'surface water': {\n",
    "        'number': 1,\n",
    "        'color': 'aec3d4'\n",
    "    },\n",
    "    'snow and ice': {\n",
    "        'number': 2,\n",
    "        'color': 'b1f9ff'\n",
    "    },\n",
    "    'mangroves': {\n",
    "        'number': 3,\n",
    "        'color': '111149'\n",
    "    },\n",
    "    'flooded forest': {\n",
    "        'number': 4,\n",
    "        'color': '287463'\n",
    "    },\n",
    "    'Deciduous forest': {\n",
    "        'number': 5,\n",
    "        'color': '152106'\n",
    "    },\n",
    "    'Orchard or plantation forest': {\n",
    "        'number': 6,\n",
    "        'color': 'c3aa69'\n",
    "    },\n",
    "    'evergreen Broadleaf': {\n",
    "        'number': 7,\n",
    "        'color': '7db087'\n",
    "    },\n",
    "    'mixed forest': {\n",
    "        'number': 8,\n",
    "        'color': '387242'\n",
    "    },\n",
    "    'urban and built up': {\n",
    "        'number': 9,\n",
    "        'color': 'cc0013'\n",
    "    },\n",
    "    'cropland': {\n",
    "        'number': 10,\n",
    "        'color': '8dc33b'\n",
    "    },\n",
    "    'rice': {\n",
    "        'number': 11,\n",
    "        'color': 'ffff00'\n",
    "    },\n",
    "    'mining': {\n",
    "        'number': 12,\n",
    "        'color': 'cec2a5'\n",
    "    },\n",
    "    'barren': {\n",
    "        'number': 13,\n",
    "        'color': '674c06'\n",
    "    },\n",
    "    'wetlands': {\n",
    "        'number': 14,\n",
    "        'color': '3bc3b2'\n",
    "    },\n",
    "    'grassland': {\n",
    "        'number': 15,\n",
    "        'color': 'f4a460'\n",
    "    },\n",
    "    'shrubland': {\n",
    "        'number': 16,\n",
    "        'color': '800080'\n",
    "    },\n",
    "    'aquaculture': {\n",
    "        'number': 17,\n",
    "        'color': '51768e'\n",
    "    }\n",
    "}\n",
    "\n",
    "# Function to get a list of ids (keys) from a structure.\n",
    "def getIds(struct):\n",
    "    return Object.keys(struct)\n",
    "\n",
    "\n",
    "# Function to replace spaces with underscores in a list of strings.\n",
    "def cleanList(list):\n",
    "\n",
    "def func_nzj(name):\n",
    "        return name.replace(/\\s+/g, '_')\n",
    "\n",
    "    return list.map(func_nzj)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Function to get a list of column values from a structure.\n",
    "def getList(struct, column):\n",
    "\n",
    "def func_dkz(k):\n",
    "        value = struct[k][column]\n",
    "        return value\n",
    "\n",
    "    return Object.keys(struct).map(func_dkz)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "classNamesList = getIds(classStruct)\n",
    "probNames = cleanList(classNamesList)\n",
    "classNames = ee.List(classNamesList)\n",
    "classNumbers = getList(classStruct, 'number')\n",
    "paletteList = getList(classStruct, 'color')\n",
    "PALETTE = paletteList.join(',')\n",
    "\n",
    "# JSON dictionary that defines piechart colors based on the\n",
    "# landcover class palette.\n",
    "# https:#developers.google.com/chart/interactive/docs/gallery/piechart\n",
    "colors = []\n",
    "for i in range(0, paletteList.length, 1):\n",
    "    colors.push({\n",
    "        'color': '_'.replace('_', paletteList[i])\n",
    "    })\n",
    "\n",
    "\n",
    "# Set start and end years.\n",
    "startYear = 2010\n",
    "endYear = 2020\n",
    "\n",
    "# Create two date objects for start and end years.\n",
    "startDate = ee.Date.fromYMD(startYear, 1, 1)\n",
    "endDate = ee.Date.fromYMD(endYear + 1, 1, 1)\n",
    "\n",
    "# Make a list with years.\n",
    "years = ee.List.sequence(startYear, endYear)\n",
    "\n",
    "# Make a list with months.\n",
    "months = ee.List.sequence(1, 12)\n",
    "\n",
    "# Import and filter the MOD13 dataset.\n",
    "mod13 = ee.ImageCollection('MODIS/006/MOD13A1')\n",
    "mod13 = mod13.filterDate(startDate, endDate)\n",
    "\n",
    "# Select the EVI.\n",
    "EVI = mod13.select('EVI')\n",
    "\n",
    "# Import and filter the MODIS Terra surface reflectance dataset.\n",
    "mod09 = ee.ImageCollection('MODIS/006/MOD09A1')\n",
    "mod09 = mod09.filterDate(startDate, endDate)\n",
    "\n",
    "landcover = ee.Image(\n",
    "        'projects/gee-book/assets/A2-5/RLCMSv3/Mekonglandcover2018') \\\n",
    "    .select('lc').clip(mekongBasin)\n",
    "\n",
    "lcVis = {\n",
    "    'palette': PALETTE,\n",
    "    'min': 0,\n",
    "    'max': classNamesList.length - 1\n",
    "}\n",
    "Map.addLayer(landcover, lcVis, '2018 Land Cover')\n",
    "\n",
    "# We use a function to remove clouds and cloud shadows.\n",
    "# We map over the mod09 image collection and select the StateQA band.\n",
    "# We mask pixels and return the image with clouds and cloud shadows masked.\n",
    "\n",
    "def func_hao(image):\n",
    "    quality = image.select('StateQA')\n",
    "    mask = image.And(quality.bitwiseAnd(1).eq(\n",
    "            0)) # No clouds. \\\n",
    "        .And(quality.bitwiseAnd(2).eq(0)); \n",
    "\n",
    "    return image.updateMask(mask)\n",
    "\n",
    "mod09 = mod09.map(func_hao)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# We use a function to calculate the Moisture Stress Index.\n",
    "# We map over the mod09 image collection and select the NIR and SWIR bands\n",
    "# We set the timestamp and return the MSI.\n",
    "\n",
    "def func_lru(image):\n",
    "    nirband = image.select('sur_refl_b02')\n",
    "    swirband = image.select('sur_refl_b06')\n",
    "    msi = swirband.divide(nirband) \\\n",
    "        .rename('MSI') \\\n",
    "        .set('system:time_start', image.get(\n",
    "            'system:time_start'))\n",
    "    return msi\n",
    "\n",
    "MSI = mod09.map(func_lru)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# We apply a nested loop where we first iterate over\n",
    "# the relevant years and then iterate over the relevant\n",
    "# months. The function returns an image with the MSI and EVI\n",
    "# for each month. A flatten is applied to convert an\n",
    "# collection of collections into a single collection.\n",
    "monthlyIndices = ee.ImageCollection.fromImages(\n",
    "\n",
    "def func_wsp(y):\n",
    "        return months.map(function(m) {\n",
    "\n",
    "            # Calculate EVI.\n",
    "            evi = EVI.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .mean() \\\n",
    "                .multiply(0.0001)\n",
    "\n",
    "            # Calculate MSI.\n",
    "            msi = MSI.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .mean()\n",
    "\n",
    "            # Return an image with all images as bands.\n",
    "            return evi.addBands(msi) \\\n",
    "                .set('year', y) \\\n",
    "                .set('month', m) \\\n",
    "                .set('system:time_start', ee.Date \\\n",
    "                    .fromYMD(y, m, 1))\n",
    "\n",
    "        })\n",
    "\n",
    "    years.map(func_wsp\n",
    ").flatten()\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    ").flatten()\n",
    ")\n",
    "\n",
    "# We select the landcover types for evergreen, deciduous forest, cropland and rice.\n",
    "evergreenForest = landcover.eq(7)\n",
    "deciduousForest = landcover.eq(5)\n",
    "cropland = landcover.eq(10)\n",
    "rice = landcover.eq(11)\n",
    "\n",
    "# Mask pixels that do not belong to the category.\n",
    "\n",
    "def func_yqe(img):\n",
    "    return img.updateMask(evergreenForest)\n",
    "\n",
    "evergreenIndex = monthlyIndices.map(func_yqe)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Mask pixels that do not belong to the category.\n",
    "\n",
    "def func_cip(img):\n",
    "    return img.updateMask(deciduousForest)\n",
    "\n",
    "deciduousIndex = monthlyIndices.map(func_cip)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Mask pixels that do not belong to the category.\n",
    "\n",
    "def func_vrq(img):\n",
    "    return img.updateMask(cropland)\n",
    "\n",
    "croplandIndex = monthlyIndices.map(func_vrq)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Mask pixels that do not belong to the category.\n",
    "\n",
    "def func_jit(img):\n",
    "    return img.updateMask(rice)\n",
    "\n",
    "riceIndex = monthlyIndices.map(func_jit)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Define the chart and print it to the console.\n",
    "chartIndices =\n",
    "    ui.Chart.image.series({\n",
    "        'imageCollection': deciduousIndex.select(['EVI', 'MSI']),\n",
    "        'region': mekongBasin,\n",
    "        'reducer': ee.Reducer.mean(),\n",
    "        'scale': 1000,\n",
    "        'xProperty': 'system:time_start'\n",
    "    }) \\\n",
    "    .setSeriesNames(['EVI', 'MSI']) \\\n",
    "    .setOptions({\n",
    "        'title': 'Monthly deciduous forest indices',\n",
    "        'hAxis': {\n",
    "            'title': 'Date',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'vAxis': {\n",
    "            'title': 'Index',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'lineWidth': 1,\n",
    "        'colors': ['darkgreen', 'brown'],\n",
    "        'curveType': 'function'\n",
    "    })\n",
    "\n",
    "# Print the chart.\n",
    "print(chartIndices)\n",
    "\n",
    "# Define the chart and print it to the console.\n",
    "chartIndices =\n",
    "    ui.Chart.image.series({\n",
    "        'imageCollection': evergreenIndex.select(['EVI', 'MSI']),\n",
    "        'region': mekongBasin,\n",
    "        'reducer': ee.Reducer.mean(),\n",
    "        'scale': 1000,\n",
    "        'xProperty': 'system:time_start'\n",
    "    }) \\\n",
    "    .setSeriesNames(['EVI', 'MSI']) \\\n",
    "    .setOptions({\n",
    "        'title': 'Monthly deciduous forest indices',\n",
    "        'hAxis': {\n",
    "            'title': 'Date',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'vAxis': {\n",
    "            'title': 'Index',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'lineWidth': 1,\n",
    "        'colors': ['darkgreen', 'brown'],\n",
    "        'curveType': 'function'\n",
    "    })\n",
    "\n",
    "# Print the chart.\n",
    "print(chartIndices)\n",
    "\n",
    "# Define the chart and print it to the console.\n",
    "chartIndices =\n",
    "    ui.Chart.image.series({\n",
    "        'imageCollection': croplandIndex.select(['EVI', 'MSI']),\n",
    "        'region': mekongBasin,\n",
    "        'reducer': ee.Reducer.mean(),\n",
    "        'scale': 1000,\n",
    "        'xProperty': 'system:time_start'\n",
    "    }) \\\n",
    "    .setSeriesNames(['EVI', 'MSI']) \\\n",
    "    .setOptions({\n",
    "        'title': 'Monthly cropland indices',\n",
    "        'hAxis': {\n",
    "            'title': 'Date',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'vAxis': {\n",
    "            'title': 'Index',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'lineWidth': 1,\n",
    "        'colors': ['darkgreen', 'brown'],\n",
    "        'curveType': 'function'\n",
    "    })\n",
    "\n",
    "# Print the chart.\n",
    "print(chartIndices)\n",
    "\n",
    "# Define the chart and print it to the console.\n",
    "chartIndices =\n",
    "    ui.Chart.image.series({\n",
    "        'imageCollection': riceIndex.select(['EVI', 'MSI']),\n",
    "        'region': mekongBasin,\n",
    "        'reducer': ee.Reducer.mean(),\n",
    "        'scale': 1000,\n",
    "        'xProperty': 'system:time_start'\n",
    "    }) \\\n",
    "    .setSeriesNames(['EVI', 'MSI']) \\\n",
    "    .setOptions({\n",
    "        'title': 'Monthly rice indices',\n",
    "        'hAxis': {\n",
    "            'title': 'Date',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'vAxis': {\n",
    "            'title': 'Index',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'lineWidth': 1,\n",
    "        'colors': ['darkgreen', 'brown'],\n",
    "        'curveType': 'function'\n",
    "    })\n",
    "\n",
    "# Print the chart.\n",
    "print(chartIndices)\n",
    "\n",
    "# Create the panel for the legend items.\n",
    "legend = ui.Panel({\n",
    "    'style': {\n",
    "        'position': 'bottom-left',\n",
    "        'padding': '8px 15px'\n",
    "    }\n",
    "})\n",
    "\n",
    "# Create and add the legend title.\n",
    "legendTitle = ui.Label({\n",
    "    'value': 'Legend',\n",
    "    'style': {\n",
    "        'fontWeight': 'bold',\n",
    "        'fontSize': '18px',\n",
    "        'margin': '0 0 4px 0',\n",
    "        'padding': '0'\n",
    "    }\n",
    "})\n",
    "\n",
    "# Creates and styles 1 row of the legend.\n",
    "def makeRow(color, name):\n",
    "    # Create the label that is actually the colored box.\n",
    "    colorBox = ui.Label({\n",
    "        'style': {\n",
    "            'backgroundColor': '#' + color,\n",
    "            # Use padding to give the box height and width.\n",
    "            'padding': '8px',\n",
    "            'margin': '0 0 4px 0'\n",
    "        }\n",
    "    })\n",
    "\n",
    "    # Create the label filled with the description text.\n",
    "    description = ui.Label({\n",
    "        'value': name,\n",
    "        'style': {\n",
    "            'margin': '0 0 4px 6px'\n",
    "        }\n",
    "    })\n",
    "\n",
    "    return ui.Panel({\n",
    "        'widgets': [colorBox, description],\n",
    "        'layout': ui.Panel.Layout.Flow('horizontal')\n",
    "    })\n",
    "\n",
    "\n",
    "legend.add(legendTitle)\n",
    "for i in range(0, classNamesList.length, 1):\n",
    "    legend.add(makeRow(paletteList[i], classNamesList[i]))\n",
    "\n",
    "\n",
    "# Add the legend to the map.\n",
    "Map.add(legend)\n",
    "\n",
    "# -----------------------------------------------------------------------------------\n",
    "# CHECKPOINT\n",
    "# -----------------------------------------------------------------------------------"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Display the interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map"
   ]
  }
 ],
 "metadata": {
  "anaconda-cloud": {},
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
